#!/usr/bin/python
import DB2

def database_connect():
  """
  Create a connection to the Apache Derby database.
  """
  
  database = 'YMLD'
  user = 'userid'
  password = 'password'
  conn = DB2.connect(database, user, password)
  return conn

def drop_tables(conn):
  """
  Remove the tables from the database.
  """
  
  curs = conn.cursor()
  stmt = 'DROP TABLE '
  tables = ('TRANSACTIONLOG', 'SEATS', 'SEATMAP',
    'PRICEPLAN', 'PERFORMANCES', 'PRODUCTIONS')
  
  for table in tables:
    try:
      curs.execute(stmt + table)
    except:
      print 'Failed to drop table ' + table
  curs.close()
  
# We always want to commit this transaction
  if 1:
    conn.commit()
  else:
    conn.rollback()

def insert_one(conn, seq):
  """
  Insert one row into the database.
  """
  
  curs = conn.cursor()
  stmt = """
  INSERT INTO PRODUCTIONS (Production_Title,
      Production_StartDate, Production_EndDate)
      VALUES (?, ?, ?)
  """
  curs.execute(stmt, seq)
  curs.close()

def insert_many(conn):
  """
  Insert multiple rows into the database using values from a list of lists.
  """
  
  curs = conn.cursor()
  stmt = """
  INSERT INTO Productions 
  (Production_Title, Production_StartDate, Production_EndDate)
  VALUES (?, ?, ?)"""
  
  productions = [
   ['Cats and Dogs', '2005-04-01', '2005-05-01'],
   ['Desperate Houses', '2005-05-02', '2005-06-01'],
   ['Icing the Puck', '2005-06-02', '2005-07-01'],
   ['Heroes', '2005-07-02', '2005-08-01'],
   ['Rubber Sacks', '2005-08-02', '2005-09-01'],
   ['Couched in Leather', '2005-09-02', '2005-10-01'],
   ['Backsplash', '2005-10-02', '2005-11-01'],
  ]
  
  curs.executemany(stmt, productions)
  curs.close()

def insertproductions(conn):
  """
  Insert sample data into YMLD tables.
  """
  productions = [
   ['Cats and Dogs', '2005-04-01', '2005-05-01'],
   ['Desperate Houses', '2005-05-02', '2005-06-01'],
   ['Icing the Puck', '2005-06-02', '2005-07-01'],
   ['Heroes', '2005-07-02', '2005-08-01'],
   ['Rubber Sacks', '2005-08-02', '2005-09-01'],
   ['Couched in Leather', '2005-09-02', '2005-10-01'],
   ['Backsplash', '2005-10-02', '2005-11-01'],
  ]
  prodstmt = """
    INSERT INTO Productions 
    (Production_Title, Production_StartDate, Production_EndDate)
    VALUES (?, ?, ?)
  """
  perfstmt = """
    INSERT INTO Performances
    (Production_No, Performance_Date, Performance_Time, Performance_SeatsAve)
    VALUES (?, ?, ?, ?)
  """
  seatstmt = """
    INSERT INTO Seats (Performance_No, PricePlan_No, Seat_No, Seat_Available)
    VALUES (?, ?, ?, ?)
  """

  prices = (
    79.25,
    84.00,
    99.99
  )

  pricestmt = """
    INSERT INTO PricePlan (PricePlan_Cost)
    VALUES (?)
  """

  curs = conn.cursor()
  curs.executemany(pricestmt, prices)

  for show in productions:
    curs.execute(prodstmt, show)
    prodid = lastinsertid(conn)
    for time in ('19:30:00', '19:45:00', '21:45:00'):
      curs.execute(perfstmt, (prodid, show[1], time, 1))
      perfid = int(lastinsertid(conn))
      curs.execute(seatstmt, (perfid, 1, 1, 1))
      curs.execute(seatstmt, (perfid, 1, 2, 0))
      curs.execute(seatstmt, (perfid, 3, 3, 1))
  curs.close()

def lastinsertid(conn):
  """
  Return the ID of the last row that was inserted.
  """
  
  curs = conn.cursor()
  stmt = """
  VALUES identity_val_local()
  """
  curs.execute(stmt)
  row = curs.fetchone()
  return row[0]
  curs.close()

def select_next_show(conn):
  """
  Select the next show from the Productions table, based on
  today's date. If there are no future shows starting after
  today, return None.
  """
  
  curs = conn.cursor()
  stmt = """
  SELECT Production_No, Production_Title,
    Production_StartDate, Production_EndDate
    FROM Productions
    WHERE Production_StartDate > CURRENT_DATE
    ORDER BY Production_StartDate
  """
 
  curs.execute(stmt)
  row = curs.fetchone()
  curs.close()
  return row

def select_all_shows(conn):
  """
  Select the next show from the Productions table, based on
  today's date. If there are no future shows starting after
  today, return None.
  """
  
  curs = conn.cursor()
  stmt = """
  SELECT Production_No, Production_Title,
    Production_StartDate, Production_EndDate
    FROM Productions
    ORDER BY Production_StartDate
  """
 
  curs.execute(stmt)
  rows = curs.fetchall()
  curs.close()
  return rows

def call_procedure_double_in(conn):
  """
  Call a procedure that only accepts IN parameters.
  """
  
  curs = conn.cursor()
  stmt = 'double_in'
  old = 'Cats and Dogs'
  new = 'My New Title'
  curs.callproc(stmt, old, new)
  curs.close()
    
def call_procedure_in(conn):
  """
  Call a procedure that only accepts IN parameters.
  """
  
  curs = conn.cursor()
  stmt = 'SQLPROCEDURES'
  parm1 = None
  parm2 = 'SYSIBM'
  parm3 = 'SQLPROCEDURES'
  parm4 = None
  parms = (parm1, parm2, parm3, parm4)
  curs.callproc(stmt, (parms))
  rows = curs.fetchall()
  curs.close()
  return rows
    
def call_procedure_inout(conn):
  """
  Call a procedure that accepts one INOUT DATE parameter.
  """
  
  curs = conn.cursor()
  stmt = 'PRODUCTION_LISTS2'
  parm = '2004-12-02'
  res = curs.callproc(stmt, parm)
  curs.close()
  return res

def call_procedure_inout_varchar(conn):
  """
  Call a procedure that returns one OUT VARCHAR parameter.
  """
  
  curs = conn.cursor()
  stmt = 'TEST_OUT_VARCHAR'
  parm = None
  res = curs.callproc(stmt, parm)
  curs.close()
  return res
  
def call_procedure_single_result_set(conn):
  """
  Call a procedure that returns a single result set.
  """
  
  curs = conn.cursor()
  stmt = 'SINGLE_RESULT'
  curs.callproc(stmt)
  rows = curs.fetchall()
  curs.close()
  return rows
    
def call_procedure_results(conn):
  """
  Call a procedure that returns three result sets.
  Just print the contents of each result set.
  """
  
  curs = conn.cursor()
  stmt = 'PRODUCTION_LISTS'
  curs.callproc(stmt)
  print curs.fetchall()
  while curs.nextset():
    print curs.fetchall()
  curs.close()

def insert_seat_view(conn, filename):
  """
  Insert a picture (BLOB) into the database from a file.
  """
  
  curs = conn.cursor()
  stmt = """
  INSERT INTO SeatMap(Seat_View)
  VALUES (?)
  """
  curs.execute(stmt, DB2.BLOB(filename))
  curs.close()

def retrieve_blob(conn, filename):
  """
  Save a BLOB from the database into a file.
  """
  
  curs = conn.cursor()
  stmt = """
  SELECT Seat_View FROM SeatMap FETCH FIRST 1 ROW ONLY
  """
  row = curs.fetchone()
  pic_data = str(row[0])
  curs.close()
  
  output = file(filename, 'wb')
  output.write(pic_data)
  output.close()
  
def select_performance_data(conn, production):
  """
  Get a list of all performances for a given production
  that still have seats available.
  """
  
  curs = conn.cursor()
  stmt = """
    SELECT Performance_No, Production_Title,
    Performance_Date, Performance_Time
    FROM Productions INNER JOIN Performances
    ON Productions.Production_No = Performances.Production_No
    WHERE Productions.Production_No = ?
    AND Performances.Performance_SeatsAve = 1
  """
  
  curs.execute(stmt, production)
  performances = curs.fetchall()
  curs.close()
  return performances

def select_available_seats(conn, performances):
  """
  Get a list of all available seats and prices for a
  given performance of a show.
  """

  curs = conn.cursor()
  stmt = """
    SELECT Seat_No, CAST(PricePlan_Cost AS DECIMAL(5, 2))
    FROM Seats INNER JOIN PricePlan
    ON Seats.PricePlan_No = PricePlan.PricePlan_No
    WHERE Performance_No = ?
    AND Seats.Seat_Available = 1
  """
  
  seats = {}
  for show in performances:
    seats[str(show[0])] = []
    curs.execute(stmt, show[0])
    row = curs.fetchone()
    while row:
      seats[str(show[0])].append((row[0], row[1]))
      row = curs.fetchone()
  curs.close()
  return seats

  
def test():
  conn = database_connect()
  print conn.autocommit()
  print select_all_shows(conn)
    
if __name__ == '__main__':
    test()
