import Tkinter as Tk
import YMLD
import DB2

class Administration:
  """
  Create an administrative GUI for the YMLD application.
  """
  def __init__(self, root):
    self.root = root
    self.conn = YMLD.database_connect()
    root.title("YourMamaLovesDrama Administration Panel")
    menubar = Tk.Menu(self.root)
    
    # Menu items
    filemenu = Tk.Menu(menubar, tearoff=0)
    menubar.add_cascade(label="File", menu=filemenu)
    filemenu.add_command(label="Refresh", command=self.refresh)
    filemenu.add_command(label="Exit", command=self.exit)
    
    productions = Tk.Menu(menubar, tearoff=0)
    menubar.add_cascade(label="Productions", menu=productions)
    productions.add_command(label="Add", command=self.addproduction)
    
    self.root.config(menu=menubar)
    
    self.display_productions()

  def display_productions(self):
    """
    Display the productions in a table in the main window.
    """
    self.frame = Tk.Frame(root)
    self.frame.pack()

    # Display all productions on main window
    columns = ('ID', 'Title', 'Start Date', 'End Date')
    y = 0
    for header in columns:
      c = Tk.Button(self.frame, text=header, font='Helvetica 10 bold')
      c.grid(row=0, column=y, sticky=Tk.E+Tk.W)
      y = y + 1
    shows = YMLD.select_all_shows(self.conn)
    x = 1
    for row in shows:
      y = 0
      for col in row:
        c = Tk.Button(self.frame, text=col, font='Helvetica 10')
        c.grid(row=x, column=y, sticky=Tk.E+Tk.W)
        y = y + 1
      x = x + 1

  def refresh(self):
    """
    Refresh the main window with the latest production
    data from the YMLD database.
    """
    self.frame.destroy()
    self.display_productions()

  def exit(self):
    """
    Quit the YMLD administration application, cleaning up
    all resources.
    """
    self.conn.close()
    self.root.quit()
    self.root.destroy()

  def addproduction(self):
    """
    Create the pop-up AddProduction dialog.
    """
    add = AddProduction(self, self.root)

class AddProduction(Tk.Toplevel):
  """
  Pop-up dialog that disables the main parent window and enables
  users to type in the information for a new YMLD production.
  """
  def __init__(self, parent, root):
    Tk.Toplevel.__init__(self, root)
    self.transient(root)
    self.root = root
    self.parent = parent
    self.title('Add Production')
        
    bottom = Tk.Frame(self)
    bottom.pack(side=Tk.BOTTOM)
    
    top = Tk.Frame(self)
    top.pack(side=Tk.TOP)
    
    labels = Tk.Frame(top)
    labels.pack(side=Tk.LEFT)
    
    entries = Tk.Frame(top)
    entries.pack(side=Tk.RIGHT)
    
    titlelabel = Tk.Label(labels, text='Title:')
    titlelabel.pack(anchor=Tk.E)
    self.titleentry = Tk.Entry(entries)
    self.titleentry.insert(0, 'Title')
    self.titleentry.pack(anchor=Tk.W)
    
    startlabel = Tk.Label(labels, text='Start date:')
    startlabel.pack(anchor=Tk.E)
    self.startentry = Tk.Entry(entries)
    self.startentry.insert(0, 'YYYY-MM-DD')
    self.startentry.pack(anchor=Tk.W)
    
    endlabel = Tk.Label(labels, text='End date:')
    endlabel.pack(anchor=Tk.E)
    self.endentry = Tk.Entry(entries)
    self.endentry.insert(0, 'YYYY-MM-DD')
    self.endentry.pack(anchor=Tk.W)
    
    # Add standard OK/Cancel buttons
    ok = Tk.Button(bottom, text='OK', command=self.addproduction)
    ok.pack(side=Tk.LEFT)
    cancel = Tk.Button(bottom, text='Cancel', command=self.cancel)
    cancel.pack(side=Tk.RIGHT)
    
    # Prevent root window from operating
    self.grab_set()
    self.initial_focus = self.titleentry
    self.initial_focus.focus_set()
    self.wait_window(root)

  def addproduction(self):
    """
    Add one new production to the YMLD database.
    """
    YMLD.insert_one(self.parent.conn, (self.titleentry.get(), \
      self.startentry.get(), self.endentry.get()))
    self.parent.refresh()
    self.cancel()
  
  def cancel(self):
    """
    Standard method for cancelling an operation and
    removing the dialog that contained the Cancel button.
    """
    self.root.focus_set()
    self.destroy()

if __name__ == '__main__':
  root = Tk.Tk()
  app = Administration(root)
  root.mainloop()


